==> clj-redis-map-0.1.19-20160726.102829-3.jar: selmeci/clojure/lang/redis_persistent_map.clj <==
(ns selmeci.clojure.lang.redis-persistent-map
  (:require [taoensso.carmine :as car :refer (wcar)]
            [clojure.core.async :as async :refer [<! chan go >!! go-loop]]
            [liberator.representation])
  (:import [clojure.lang MapEntry]
           [liberator.representation RingResponse]
           [java.util.AbstractMap$SimpleEntry]))

(defn set-command [server_ops lock k v]
  (fn []
    (wcar server_ops (car/set k (car/freeze v)))))

(defn del-command [server_ops lock k]
  (fn []
    (wcar server_ops (car/del k))))

(defprotocol RedisServerOps
  (server-ops [this])
  (key-prefix [this]))

(deftype RedisPersistentMap [server_ops lock key_prefix modification_chan]
  RedisServerOps
  (server-ops [this]
    server_ops)
  (key-prefix [this]
    key_prefix)
  
  clojure.lang.IFn
  (invoke [this k]
    (wcar server_ops (car/get (car/key key_prefix k))))
  
  (invoke [this k not_found]
    (if-let [v (wcar server_ops (car/get (car/key key_prefix k)))]
      v
      not_found))
  
  clojure.lang.IPersistentMap
  (assoc [this k v]
    (>!! modification_chan (set-command server_ops lock (car/key key_prefix k) v))
    this)
  (assocEx [this k v]
    (if (= 1 (wcar server_ops (car/exists (car/key key_prefix k))))
      (throw (java.lang.RuntimeException. "Key already present"))
      (>!! modification_chan (set-command server_ops lock (car/key key_prefix k) v)))
    this)
  (without [this k]
    (>!! modification_chan (del-command server_ops lock (car/key key_prefix k)))
    this)
  
  java.lang.Iterable
  (iterator [this]
    (.iterator
      (map (fn [k] 
             (java.util.AbstractMap$SimpleEntry. (keyword (second (clojure.string/split k #":")))
                                                 (wcar server_ops (car/get k))))
           (wcar server_ops (car/keys (car/key key_prefix "*"))))))
  
  clojure.lang.Associative
  (containsKey [this k]
    (= 1 (wcar server_ops (car/exists (car/key key_prefix k)))))
  (entryAt [this k]
    (MapEntry. k (wcar server_ops (car/get (car/key key_prefix k)))))
  
  clojure.lang.IPersistentCollection
  (count [_]
    (clojure.core/count (wcar server_ops (car/keys (car/key key_prefix "*")))))
  (cons [this [k v]]
    (>!! modification_chan (set-command server_ops lock (car/key key_prefix k) v))
    this)
  (empty [this]
    (let [rm_keys (wcar server_ops (car/keys (car/key key_prefix "*")))
          rm (partial apply car/del rm_keys)]
      (wcar server_ops (rm)))
    this)
  (equiv [this o]
    (and (isa? (class o) RedisPersistentMap)
         (.equiv server_ops (server-ops o))
         (= key_prefix (:key_prefix o))))
  
  clojure.lang.Seqable
  (seq [this]
    (.seq 
      (map (fn [k] 
             (java.util.AbstractMap$SimpleEntry. (keyword (second (clojure.string/split k #":")))
                                                 (wcar server_ops (car/get k))))
           (wcar server_ops (car/keys (car/key key_prefix "*"))))))
  
  clojure.lang.ILookup
  (valAt [this k]
    (wcar server_ops (car/get (car/key key_prefix k))))
  (valAt [this k not_found]
    (if-let [v (wcar server_ops (car/get (car/key key_prefix k)))]
      v
      not_found)))

(defn create->RedisPersistentMap [server_ops lock key_prefix]
  (let [modification_chan (chan )]
    (go-loop [cmd (<! modification_chan)]
             (if-not (nil? cmd)
               (do
                 (cmd )
                 (recur (<! modification_chan)))))
    (->RedisPersistentMap server_ops lock key_prefix modification_chan)))

==> clj-redis-map-0.1.21-20161109.112918-5.jar: selmeci/clojure/lang/redis_persistent_map.clj <==
(ns selmeci.clojure.lang.redis-persistent-map
  (:require [taoensso.carmine :as car :refer (wcar)]
            [liberator.representation])
  (:import [clojure.lang MapEntry]
           [liberator.representation RingResponse]
           [java.util.AbstractMap$SimpleEntry]))

(defn set-command [server_ops k v]
  (wcar server_ops (car/set k (car/freeze v))))

(defn del-command [server_ops k]
  (wcar server_ops (car/del k)))

(defprotocol RedisServerOps
  (server-ops [this])
  (key-prefix [this]))

(deftype RedisPersistentMap [server_ops key_prefix]
  RedisServerOps
  (server-ops [this]
    server_ops)
  (key-prefix [this]
    key_prefix)
  
  clojure.lang.IFn
  (invoke [this k]
    (wcar server_ops (car/get (car/key key_prefix k))))
  
  (invoke [this k not_found]
    (if-let [v (wcar server_ops (car/get (car/key key_prefix k)))]
      v
      not_found))
  
  clojure.lang.IPersistentMap
  (assoc [this k v]
    (set-command server_ops (car/key key_prefix k) v)
    this)
  (assocEx [this k v]
    (if (= 1 (wcar server_ops (car/exists (car/key key_prefix k))))
      (throw (java.lang.RuntimeException. "Key already present"))
      (set-command server_ops (car/key key_prefix k) v))
    this)
  (without [this k]
    (del-command server_ops (car/key key_prefix k))
    this)
  
  java.lang.Iterable
  (iterator [this]
    (.iterator
      (map (fn [k] 
             (java.util.AbstractMap$SimpleEntry. (keyword (second (clojure.string/split k #":")))
                                                 (wcar server_ops (car/get k))))
           (wcar server_ops (car/keys (car/key key_prefix "*"))))))
  
  clojure.lang.Associative
  (containsKey [this k]
    (= 1 (wcar server_ops (car/exists (car/key key_prefix k)))))
  (entryAt [this k]
    (MapEntry. k (wcar server_ops (car/get (car/key key_prefix k)))))
  
  clojure.lang.IPersistentCollection
  (count [_]
    (clojure.core/count (wcar server_ops (car/keys (car/key key_prefix "*")))))
  (cons [this [k v]]
    (set-command server_ops (car/key key_prefix k) v)
    this)
  (empty [this]
    (let [rm_keys (wcar server_ops (car/keys (car/key key_prefix "*")))
          rm (partial apply car/del rm_keys)]
      (wcar server_ops (rm)))
    this)
  (equiv [this o]
    (and (isa? (class o) RedisPersistentMap)
         (.equiv server_ops (server-ops o))
         (= key_prefix (:key_prefix o))))
  
  clojure.lang.Seqable
  (seq [this]
    (.seq 
      (map (fn [k] 
             (java.util.AbstractMap$SimpleEntry. (keyword (second (clojure.string/split k #":")))
                                                 (wcar server_ops (car/get k))))
           (wcar server_ops (car/keys (car/key key_prefix "*"))))))
  
  clojure.lang.ILookup
  (valAt [this k]
    (wcar server_ops (car/get (car/key key_prefix k))))
  (valAt [this k not_found]
    (if-let [v (wcar server_ops (car/get (car/key key_prefix k)))]
      v
      not_found)))
